package org.artifactory.queue.services;

import org.artifactory.queue.QueueConfiguration;

public interface PersistentQueue extends QueueOperations {

    /**
     * entry point for the real work of the queue is done here.
     */
    TriggerResult trigger();

    /**
     * signal queue to shut down, stop all running jobs
     */
    void shutdown();

    String getName();

    String getRepoKey();

    QueueConfiguration getQueueConfiguration();

    /**
     * 1. cleanup existing errors
     * 2. reset queue marker to last event
     * OR
     * delete marker related to the queue
     *
     * @param repoKey - the repo key
     * @param delete true - delete marker, false - set to last event.
     */
    void reset(String repoKey, boolean delete);

    /**
     * Create cursor marker if not exists
     */
    void createMarkerIfNotExists();

    enum TriggerResult {
        /**
         * All events were processed successfully
         */
        FINISHED_ALL,
        /**
         * Events were process and finished with a handled QueueEventException
         * we can do more retires on this status because backoffStrategy can prevent infinite loops
         */
        FINISHED_WITH_HANDLED_ERROR,
        /**
         * Events were processed and finished with an unexpected error
         * we will not create retires for this status because there is no backoffStrategy to prevent infinite loops
         */
        FINISHED_WITH_UNHANDLED_ERROR,
        /**
         * no events found
         */
        NOTHING_TO_DO,
        /**
         * queue was shutdown or paused
         */
        NOT_AVAILABLE
    }
}
