package org.artifactory.servicetrust;

import java.io.IOException;
import java.util.Optional;

/**
 * @author igoru
 */
public interface ServiceTrustService {

    /**
     * Creates a pairing token
     *
     * @param useCasePath     use case value
     * @param trust       use case trust
     * @param baseUrl  base url value
     * @param pairingUrl  pairing url value
     * @param exchangeUrl exchange url value
     * @return registration token value
     */
    String getPairingToken(
            String useCasePath, ServiceTrust trust, String baseUrl, String pairingUrl, String exchangeUrl);

    /**
     * @throws IOException
     */
    void pairToken(String useCasePath, ServiceTrustUseCase serviceTrustUseCase, String token) throws IOException;

    /**
     * Exchanges pairing token with a master token
     *
     * @param request exchange request
     * @return master token information
     */
    ExchangeTokenResponse exchangeToken(ExchangeTokenRequest request);

    /**
     * Finds use case in the list of registered use cases
     *
     * @param useCasePath use case value
     * @return if case such use case is known returns optional with use case value, empty optional otherwise
     */
    Optional<ServiceTrustUseCase> findUseCase(String useCasePath);

    /**
     * Refreshes a master token
     *
     * @param exchangeUrl exchange url value
     * @param masterToken active master token
     */
    void refreshMasterToken(ServiceTrustUseCase serviceTrustUseCase, String exchangeUrl, String masterToken)
            throws IOException;

    /**
     * Exchange a master token to a scoped token
     *
     * @param serviceTrustUseCase - use case
     * @param exchangeUrl         - exchange url
     * @param masterToken         - master token
     * @return scoped token info
     * @throws IOException
     */
    ExchangeTokenResponse exchangeMasterToken(ServiceTrustUseCase serviceTrustUseCase, String exchangeUrl,
            String masterToken) throws IOException;
}
