package org.artifactory.security.access;

import com.jfrog.sysconf.SysConfig;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.test.TestUtils;
import org.artifactory.util.ListeningPortDetector;
import org.jfrog.access.common.ServiceId;
import org.jfrog.access.router.registration.HealthCheckEndpoint;
import org.jfrog.access.router.registration.LocalEndpoint;
import org.jfrog.access.router.registration.RoutePath;
import org.jfrog.access.router.registration.RouterRegistrationData;
import org.testng.annotations.Test;

import java.util.List;

import static org.fest.assertions.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Tamir Hadad
 */
@Test
public class ArtifactoryRouterRegistrationDataTest extends ArtifactoryHomeBoundTest {

    public void testRouterRegistrationDataBuilder() {
        int port = 9090;
        SysConfig sysConfig = mock(SysConfig.class);
        SysConfig sysConfigBackUp = TestUtils.getField(ArtifactoryHome.get(), "sysConfig", SysConfig.class);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);

        when(sysConfig.getInt(eq(ListeningPortDetector.SYS_MAINTENANCE_ARTIFACTORY_PORT), anyInt()))
                .thenReturn(port);

        ServiceId serviceId = new ServiceId("jf", "test");

        RouterRegistrationData registrationData = new ArtifactoryRouterRegistrationData(serviceId, "nodeId", "test")
                .build();
        assertThat(registrationData.getNodeId()).isEqualTo("nodeId");
        assertThat(registrationData.getServiceId()).isEqualTo(serviceId);
        List<LocalEndpoint> endpoints = registrationData.getEndpoints();
        assertThat(endpoints).hasSize(1);
        List<RoutePath> routePaths = endpoints.get(0).getRoutePaths();
        assertThat(routePaths).hasSize(3).containsExactly(
                new RoutePath("/artifactory"), new RoutePath("/artifactory/(.*)"), new RoutePath("/v2/(.*)"));

        HealthCheckEndpoint healthCheckEndpoint = registrationData.getHealthCheckEndpoint();
        assertThat(healthCheckEndpoint.getPath()).isEqualTo("/test/api/v1/system/readiness");
        assertThat(healthCheckEndpoint.getPort()).isEqualTo(port);
    }
}
