package org.artifactory.api.events;

/**
 * Add a new type here when you need to use the EventLogHintManager API.
 * Be sure to use different prefix for each type.
 * prefix is saved as part of the hint value in the node_events table e,g: "ts1.value"
 * The prefix allow multiple types with same value coexist for same event, so developer should not worry about what
 * kind of value is permitted
 *
 * @author avishayh
 */
public enum EventHintType {

    FederatedReplication("fdr"),
    RemoveRepo("rmr"),
    RollbackEventId("rei"),
    Test("ts1"),
    Test2("ts2");

    private final String prefix;

    EventHintType(String prefix) {
        assert prefix.length() == 3;
        this.prefix = prefix + ".";
    }

    public String getPrefix() {
        return prefix;
    }
}
