package org.artifactory.api.events;

import org.testng.annotations.Test;

import java.util.HashSet;
import java.util.Set;

import static org.artifactory.api.events.EventHintType.Test;
import static org.artifactory.api.events.EventHintType.Test2;
import static org.artifactory.api.events.EventLogHintManager.getHints;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

/**
 * @author avishayh
 */
public class EventLogHintManagerTest {

    @Test
    public void putAndRemoveHint() {
        assertNull(getHints(), "starting with null");
        EventLogHintManager.put(Test, "val1");
        assertNotNull(getHints(), "hints not null anymore");
        assertEquals(getHints().size(), 1, "added hint 1");
        assertTrue(getHints().contains(Test.getPrefix() + "val1"));
        EventLogHintManager.put(Test2, "val2");
        assertEquals(getHints().size(), 2, "added different hint 2");
        assertTrue(getHints().contains(Test2.getPrefix() + "val2"));
        EventLogHintManager.put(Test2, "val3");
        assertEquals(getHints().size(), 2, "try to add same hint 2 a new value");
        assertTrue(getHints().contains(Test2.getPrefix() + "val3"));
        EventLogHintManager.remove(Test2);
        assertEquals(getHints().size(), 1, "remove hint 2");
        EventLogHintManager.remove(Test2);
        assertEquals(getHints().size(), 1, "try remove same hint 2");
        EventLogHintManager.remove(Test);
        assertNull(getHints(), "last hint removed, everything is cleaned");
        EventLogHintManager.remove(Test);
        assertNull(getHints(), "calling remove after null is not failing");
    }

    @Test
    public void getValue() {
        Set<String> hints = new HashSet<>();
        hints.add(Test.getPrefix() + "value1");
        assertEquals(EventLogHintManager.getValue(hints, Test), "value1", "find value1");
        assertNull(EventLogHintManager.getValue(null, Test), "not failing for null hints");
        assertNull(EventLogHintManager.getValue(hints, Test2), "cannot find wrong hint");
        hints.add(Test2.getPrefix() + "value2");
        assertEquals(EventLogHintManager.getValue(hints, Test2), "value2", "find value for different hint");
        //check wrong values doesn't fail method
        Set<String> wrongHintsValues = new HashSet<>();
        wrongHintsValues.add("value1");
        wrongHintsValues.add(null);
        wrongHintsValues.add("");
        assertNull(EventLogHintManager.getValue(wrongHintsValues, Test), "wrong values not failing");
    }

}
