package org.artifactory.util;

import org.artifactory.exception.DocumentBuildException;
import org.jdom2.Document;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * @author Volodymyr Shulga
 */
@Test
public class XmlUtilsTest {

    @Test
    public void shouldParseSuccessfully() {
        Document document = XmlUtils
                .parse(XmlUtilsTest.class.getResourceAsStream("/config/test/config.2.2.3.with.docker.repos.xml"));
        Assert.assertNotNull(document);
    }

    @Test(expectedExceptions = {DocumentBuildException.class},
            expectedExceptionsMessageRegExp = "Error on line 2: DOCTYPE is disallowed when the feature " +
                    "\\\"http://apache.org/xml/features/disallow-doctype-decl\\\" set to true."
    )
    public void shouldFailParseForDoctype() throws IOException {
        String config = Files.readString(
                Path.of(
                        XmlUtilsTest.class.getResource("/config/test/config.3.0.2_doctype.xml").getPath()
                )
        );
        Assert.assertNotNull(config);
        XmlUtils.parseStrict(config);
    }

    @Test
    public void shouldProcessDoctypeWithoutStrictOption() {
        Document document = XmlUtils.parse(
                XmlUtilsTest.class.getResourceAsStream("/config/test/config.3.0.2_doctype.xml")
        );
        Assert.assertNotNull(document);
    }
}
